#ifndef __LISTS_H__
#define __LISTS_H__

#define L_FREGEX (1)
#define L_FDATA (1<<1)
#define L_FLABEL (1<<2)
#define L_FRLABEL (1<<3)

typedef struct _listentry {
	struct _listentry	*prev;
	struct _listentry	*next;
	char	*label;
	void	*data;
} listentry;

typedef listentry * ListNode;
typedef listentry * List;
typedef int (*compare_t)(listentry *, listentry *);

/* Creating and handling */
listentry * list_item_new (void);
listentry * list_new (void);
void list_append (listentry *rootnode, listentry *node);
void list_prepend (listentry *rootnode, listentry *node);
void list_item_destroy (listentry *tempnode, int flags);
void list_item_destroy_data (listentry *tempnode, int flags);
void list_destroy (listentry *rootnode);
long list_max_label (listentry *rootnode);
void list_remove (listentry *node);
void list_combine (listentry *list1, listentry *list2);
long list_length (listentry *root);

/* Sorting */
void list_qsort_segment (listentry *startnode, listentry *endnode,
				int (*compare)(listentry *node1, listentry *node2));
void list_sort (listentry *rootnode);

void list_qsort (listentry *rootnode,
				int (*compare)(listentry *node1, listentry *node2));
listentry *
	 list_msort_merge (listentry *root1, listentry *root2,
				int (*compare)(listentry *node1, listentry *node2));
listentry *
	 list_msort_merge_special_add (listentry *root1, listentry *root2,
				int (*compare)(listentry *node1, listentry *node2));
listentry *
	 list_msort_merge_special_combine (listentry *root1, listentry *root2,
				int (*compare)(listentry *node1, listentry *node2));

/* Comparing */
int list_data_rcompare	(listentry *node1, listentry *node2);
int list_data_compare	(listentry *node1, listentry *node2);
// int list_data_value_compare (listentry *node1, listentry *node2);
int list_label_compare	(listentry *node1, listentry *node2);
int list_number_compare	(listentry *node1, listentry *node2);
int list_number_rcompare(listentry *node1, listentry *node2);

#endif
