#include <stdio.h>
#include <netinet/in.h>
#include "value.h"
#include "lists.h"
#include "tree.h"

static char	*tree_buf = NULL;
static int	 tree_length = 0;
static int	 tree_offset = 0;

#define BUFFER_ADD(x) do { int len = strlen(x);				\
				strncpy(&tree_buf[tree_offset], x, len);	\
				tree_offset += len; } while(0);

void
tree_allocate(int length)
{
	tree_offset = 0;
	if (length >= tree_length) {
		tree_buf = (char *)realloc(tree_buf,
						(length + 10) * sizeof(char));
		tree_length = length + 10;
	}
}

void tree_print_array (FILE *stream, value *v);
void tree_print_mapping (FILE *stream, value *v);

void
tree_print_value (FILE *stream, value *v)
{
	if (!v) {
		fprintf(stream, VNUL);
		return;
	}
	switch(v->v_type) {
		case MM_ARRAY:
			if (v->v_array) {
				BUFFER_ADD("({");
				fprintf(stream, "({");
				tree_print_array(stream, v);
				BUFFER_ADD("})");
				fprintf(stream, "})");
			} else fprintf(stream, VNUL);
			break;
		case MM_MAPPING:
			if (v->v_array) {
				BUFFER_ADD("([");
				fprintf(stream, "([");
				tree_print_mapping(stream, v);
				BUFFER_ADD("])");
				fprintf(stream, "])");
			} else fprintf(stream, VNUL);
			break;
		case MM_STRING:
			if (v->v_string)
				fprintf(stream, "\"%s\"", v->v_string);
			else
				fprintf(stream, VNUL);
			break;
		case MM_VALUE:
			fprintf(stream, "%ld", v->v_value);
			break;
		case MM_NONE:
			// fprintf(stream, "[ERROR]");
			/* This allows just to print a comma blank */
			break;
	}
}

void
tree_print (FILE *stream, value *v)
{
	int	 nlen;

	if (!v) return;
	fprintf(stderr, "\nLength is %ld\n", v->v_plength);
	tree_allocate(v->v_plength);


	nlen = htonl(v->v_plength);
	fwrite(&nlen, sizeof(unsigned long), 1, stream);
	tree_print_value(stream, v);
	fflush(stream);
}

void
tree_print_array (FILE *stream, value *v)
{
	listentry	*list;
	listentry	*node;

	if (!v) return;
	list = (listentry *)(v->v_array);	/* list asserted above */
	node = list->next;
	while (node != list) {
		tree_print_value(stream, (value *)(node->data));
		fprintf(stream, ",");
		node = node->next;
	}
}

void
tree_print_mapping (FILE *stream, value *v)
{
	listentry	*list;
	listentry	*node;
	value		*d;

	list = (listentry *)(v->v_array);	/* list asserted above */
	node = list->next;
	while (node != list) {
		d = (value *)(node->data);
		tree_print_value(stream, d->k);
		fprintf(stream, ":");
		tree_print_value(stream, d->d);
		fprintf(stream, ",");
		node = node->next;
	}
}
